//*************************************************************************
//* -----------
//* CDCAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze CDC data.
//* (Update Record)
//*	2000/05/16  K.Fujii	Original version.
//*************************************************************************

#include "JSFSteer.h"
#include "CDCAnalysis.h"
#include <iostream.h>

ClassImp(CDCAnalysis)

const Int_t CDCAnalysis::kZoneX   = 4;
const Int_t CDCAnalysis::kZoneY   = 3;
const Int_t CDCAnalysis::kNh = kZoneX*kZoneY;

//___________________________________________________________________________
CDCAnalysis::CDCAnalysis(const Char_t *name, const Char_t *title)
	: JSFModule(name, title), fMode(0), fNwires(5)
{
   hdefr = new THdef2[kNh];
   hdefs = new THdef2[kNh];
   hdefp = new THdef2[kNh];
   hdefz = new THdef2[kNh];
   hdefe = new THdef1[kNh];
   hdefw = new THdef2[kNh];
   hdefg = new THdef2[kNh];
   Char_t st[256], std[256];
   Int_t i;
   for (i = 0; i < kNh; i++) {
      sprintf(st,"CDCAnalysis.hRes%2.2d",i);
      sprintf(std,"Res%2.2d 50,0.,5.,200,-0.2,0.2",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefr[i].title,
			&hdefr[i].nxbin, &hdefr[i].xmin, &hdefr[i].xmax,
			&hdefr[i].nybin, &hdefr[i].ymin, &hdefr[i].ymax);
      sprintf(st,"CDCAnalysis.hPh%2.2d",i);
      sprintf(std,"Ph%2.2d 50,0.,5.,200,0.,10000.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefs[i].title,
			&hdefs[i].nxbin, &hdefs[i].xmin, &hdefs[i].xmax,
			&hdefs[i].nybin, &hdefs[i].ymin, &hdefs[i].ymax);
      sprintf(st,"CDCAnalysis.hPhPk%2.2d",i);
      sprintf(std,"PhPk%2.2d 50,0.,5.,256,0.,256.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefp[i].title,
			&hdefp[i].nxbin, &hdefp[i].xmin, &hdefp[i].xmax,
			&hdefp[i].nybin, &hdefp[i].ymin, &hdefp[i].ymax);
      sprintf(st,"CDCAnalysis.hPed%2.2d",i);
      sprintf(std,"Ped%2.2d 50,0.,5.,256,0.,256.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefz[i].title,
			&hdefz[i].nxbin, &hdefz[i].xmin, &hdefz[i].xmax,
			&hdefz[i].nybin, &hdefz[i].ymin, &hdefz[i].ymax);
      sprintf(st,"CDCAnalysis.hEff%2.2d",i);
      sprintf(std,"Eff%2.2d 10,0.,5.",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g",
      			 hdefe[i].title,
	                &hdefe[i].nxbin, &hdefe[i].xmin, &hdefe[i].xmax);
      sprintf(st,"CDCAnalysis.hTwk%2.2d",i);
      sprintf(std,"Twk%2.2d 256,0.,256.,200,-0.2,0.2",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefw[i].title,
			&hdefw[i].nxbin, &hdefw[i].xmin, &hdefw[i].xmax,
			&hdefw[i].nybin, &hdefw[i].ymin, &hdefw[i].ymax);
      sprintf(st,"CDCAnalysis.hSgt2%2.2d",i);
      sprintf(std,"Sgt2%2.2d 50,0.,5.,100,0.,0.0001",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g,%d,%g,%g",
      			 hdefg[i].title,
			&hdefg[i].nxbin, &hdefg[i].xmin, &hdefg[i].xmax,
			&hdefg[i].nybin, &hdefg[i].ymin, &hdefg[i].ymax);
   }
   hRes    = new TH2F* [kNh];
   hPhSum  = new TH2F* [kNh];
   hPhPk   = new TH2F* [kNh];
   hPed    = new TH2F* [kNh];
   hEff    = new TH1E* [kNh];
   hTwk    = new TH2F* [kNh];
   hSgt2   = new TH2F* [kNh];
}

//___________________________________________________________________________
CDCAnalysis::~CDCAnalysis()
{
   delete [] hdefr;
   delete [] hdefs;
   delete [] hdefp;
   delete [] hdefz;
   delete [] hdefe;
   delete [] hdefw;
   delete [] hdefg;
   Int_t i;
   for (i = 0; i < kNh; i++) {
   	delete hRes  [i];
   	delete hPhSum[i];
   	delete hPhPk [i];
   	delete hPed  [i];
	delete hEff  [i];
	delete hTwk  [i];
	delete hSgt2 [i];
   }
   delete [] hRes;
   delete [] hPhSum;
   delete [] hPhPk;
   delete [] hPed;
   delete [] hEff;
   delete [] hTwk;
   delete [] hSgt2;
}

//___________________________________________________________________________
Bool_t CDCAnalysis::Initialize()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < kNh; i++) {
      hRes[i] = new TH2F(hdefr[i].title,hdefr[i].title,
      			hdefr[i].nxbin,hdefr[i].xmin,hdefr[i].xmax,
      			hdefr[i].nybin,hdefr[i].ymin,hdefr[i].ymax);
      cout << i << " "      << hdefr[i].title 
      		<< " nx "   << hdefr[i].nxbin
      		<< " xmin " << hdefr[i].xmin
      		<< " xmax " << hdefr[i].xmax
      		<< " ny "   << hdefr[i].nybin
      		<< " ymin " << hdefr[i].ymin
      		<< " ymax " << hdefr[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hPhSum[i] = new TH2F(hdefs[i].title,hdefs[i].title,
      			hdefs[i].nxbin,hdefs[i].xmin,hdefs[i].xmax,
      			hdefs[i].nybin,hdefs[i].ymin,hdefs[i].ymax);
      cout << i << " "      << hdefs[i].title 
      		<< " nx "   << hdefs[i].nxbin
      		<< " xmin " << hdefs[i].xmin
      		<< " xmax " << hdefs[i].xmax
      		<< " ny "   << hdefs[i].nybin
      		<< " ymin " << hdefs[i].ymin
      		<< " ymax " << hdefs[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hPhPk[i] = new TH2F(hdefp[i].title,hdefp[i].title,
      			hdefp[i].nxbin,hdefp[i].xmin,hdefp[i].xmax,
      			hdefp[i].nybin,hdefp[i].ymin,hdefp[i].ymax);
      cout << i << " "      << hdefp[i].title 
      		<< " nx "   << hdefp[i].nxbin
      		<< " xmin " << hdefp[i].xmin
      		<< " xmax " << hdefp[i].xmax
      		<< " ny "   << hdefp[i].nybin
      		<< " ymin " << hdefp[i].ymin
      		<< " ymax " << hdefp[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hPed[i] = new TH2F(hdefz[i].title,hdefz[i].title,
      			hdefz[i].nxbin,hdefz[i].xmin,hdefz[i].xmax,
      			hdefz[i].nybin,hdefz[i].ymin,hdefz[i].ymax);
      cout << i << " "      << hdefz[i].title 
      		<< " nx "   << hdefz[i].nxbin
      		<< " xmin " << hdefz[i].xmin
      		<< " xmax " << hdefz[i].xmax
      		<< " ny "   << hdefz[i].nybin
      		<< " ymin " << hdefz[i].ymin
      		<< " ymax " << hdefz[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hEff[i] = new TH1E(hdefe[i].title,hdefe[i].title,
      			hdefe[i].nxbin,hdefe[i].xmin,hdefe[i].xmax);
      cout << i << " "      << hdefe[i].title 
      		<< " nx "   << hdefe[i].nxbin
      		<< " xmin " << hdefe[i].xmin
      		<< " xmax " << hdefe[i].xmax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hTwk[i] = new TH2F(hdefw[i].title,hdefw[i].title,
      			hdefw[i].nxbin,hdefw[i].xmin,hdefw[i].xmax,
      			hdefw[i].nybin,hdefw[i].ymin,hdefw[i].ymax);
      cout << i << " "      << hdefw[i].title 
      		<< " nx "   << hdefw[i].nxbin
      		<< " xmin " << hdefw[i].xmin
      		<< " xmax " << hdefw[i].xmax
      		<< " ny "   << hdefw[i].nybin
      		<< " ymin " << hdefw[i].ymin
      		<< " ymax " << hdefw[i].ymax
      		<< endl;
   }
   for (i = 0; i < kNh; i++) {
      hSgt2[i] = new TH2F(hdefg[i].title,hdefg[i].title,
      			hdefg[i].nxbin,hdefg[i].xmin,hdefg[i].xmax,
      			hdefg[i].nybin,hdefg[i].ymin,hdefg[i].ymax);
      cout << i << " "      << hdefg[i].title 
      		<< " nx "   << hdefg[i].nxbin
      		<< " xmin " << hdefg[i].xmin
      		<< " xmax " << hdefg[i].xmax
      		<< " ny "   << hdefg[i].nybin
      		<< " ymin " << hdefg[i].ymin
      		<< " ymax " << hdefg[i].ymax
      		<< endl;
   }
   hcp = new TCanvas("hcp","Canvas p",20, 20, 200*kZoneX, 200*kZoneY);
   hcp->Divide(kZoneX,kZoneY);
   hcs = new TCanvas("hcs","Canvas s",20, 20, 200*kZoneX, 200*kZoneY);
   hcs->Divide(kZoneX,kZoneY);
   hcz = new TCanvas("hcz","Canvas z",20, 20, 200*kZoneX, 200*kZoneY);
   hcz->Divide(kZoneX,kZoneY);
   hcr = new TCanvas("hcr","Canvas r",20, 20, 200*kZoneX, 200*kZoneY);
   hcr->Divide(kZoneX,kZoneY);
   hce = new TCanvas("hce","Canvas e",20, 20, 200*kZoneX, 200*kZoneY);
   hce->Divide(kZoneX,kZoneY);
   hcw = new TCanvas("hcw","Canvas w",20, 20, 200*kZoneX, 200*kZoneY);
   hcw->Divide(kZoneX,kZoneY);
   hcg = new TCanvas("hcg","Canvas g",20, 20, 200*kZoneX, 200*kZoneY);
   hcg->Divide(kZoneX,kZoneY);
   last->cd();
   return kTRUE;
}


//___________________________________________________________________________
Bool_t CDCAnalysis::Process(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   CDCTrackMaker *tm = (CDCTrackMaker*)gJSF->FindModule("CDCTrackMaker");
   CDCTrackBuf   *tb = (CDCTrackBuf *)tm->EventBuf();
   CDCTrack	 *tp = (CDCTrack *)tb->GetTracks();
   TIter nexttk(tp);
   while ((tp = (CDCTrack *)nexttk())) {
      if (tp->GetEntries() < fNwires) continue;	// require fNwires wires
      CDCTrackHit *hitp;
      CDCHit *htp;
      CDCTrack trk(*tp);
      TIter next(&trk);
      Stat_t wat[CDCAnalysis::kNh];
      Int_t i;
      for (i = 0; i < CDCAnalysis::kNh; i++) wat[i] = 0;
      while ((hitp = (CDCTrackHit *)next())) {
         htp = hitp->GetCDCHit();
         Int_t layer = htp->GetLayerNo();
         Int_t cell  = htp->GetCellNo();
         Int_t wire  = htp->GetWireNo();
         Int_t a     = 5*layer + wire;		// works only for baby
         if (cell == 1) a = 10 + wire;		// works only for baby
         if (fMode == 0) { 
	 	hitp->Lock();			// lock this hit and
         	trk.Update();			// exclude it from fit
         }
         Double_t x    = trk.Xat(htp->Ywire()) - htp->Xwire();
         Double_t dx   = hitp->GetResidual();
         Double_t ph   = hitp->GetCDCHit()->GetCluster()->GetSum();
         Double_t php  = hitp->GetCDCHit()->GetCluster()->GetPeak();
         Double_t ped  = hitp->GetCDCHit()->GetCluster()->GetPedestal();
         Double_t sgx2 = trk.SGX2at(htp->Ywire());
	 cerr << "sgx2 = " << sgx2 << endl;
         if ( a >= 0 && a < CDCAnalysis::kNh ) {
      	    hRes  [a]->Fill(x,dx,1.0);
      	    hPhSum[a]->Fill(x,ph,1.0);
      	    hPhPk [a]->Fill(x,php,1.0);
      	    hPed  [a]->Fill(x,ped,1.0);
            hTwk  [a]->Fill(php,dx,1.0);
            hSgt2 [a]->Fill(x,sgx2,1.0);
	    wat[a] = 1.;
	 }
         if (fMode == 0) {
	       hitp->Unlock();	                // unlock this hit
	 }
      }
      for (i = 0; i < CDCAnalysis::kNh; i++) {
      	 Double_t x, y;
      	 Int_t layer, cell, wire;
      	 CDCHitWire::ChannelToWire(i,layer,cell,wire,x,y);
      	 hEff[i]->Fill(tp->Xat(y),wat[i]);
      }
   }
   if (!(gROOT->IsBatch()) && ev%200 == 1) Draw();
   last->cd();
   return kTRUE;
}

//___________________________________________________________________________
void CDCAnalysis::Draw(Option_t *opt)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < CDCAnalysis::kNh; i++) {
      hcp->cd(i+1); hPhPk [i]->Draw(); 	hcp->Update();
      hcs->cd(i+1); hPhSum[i]->Draw(); 	hcs->Update();
      hcz->cd(i+1); hPed  [i]->Draw(); 	hcz->Update();
      hcr->cd(i+1); hRes  [i]->Draw(); 	hcr->Update();
      hce->cd(i+1); hEff  [i]->Draw(); 	hce->Update();
      hcw->cd(i+1); hTwk  [i]->Draw();  hcw->Update();
      hcg->cd(i+1); hSgt2 [i]->Draw();  hcg->Update();
   }
   last->cd();
}



